@@ -33,10 +33,10 @@
 #include <sys/types.h>
 #include <fcntl.h>
 #include <unistd.h>
-#include <pwd.h>
 #include <sys/stat.h>
 
 #include <glib.h>
+#include <glib/gstdio.h>
 #include "gnome-i18nP.h"
 
 #include "gnome-init.h"
@@ -45,14 +45,14 @@
 #include "gnome-sound.h"
 #include "gnome-triggers.h"
 
-#include <errno.h>
-
 #include <bonobo-activation/bonobo-activation.h>
 #include <bonobo-activation/bonobo-activation-version.h>
 #include <libbonobo.h>
 
 #include <libgnomevfs/gnome-vfs-init.h>
 
+#include "libgnome-private.h"
+
 /*****************************************************************************
  * bonobo
  *****************************************************************************/
@@ -261,6 +261,25 @@ libgnome_option_cb (poptContext ctx, enum poptCallbackReason reason,
 	}
 }
 
+static int
+safe_mkdir (const char *pathname, mode_t mode)
+{
+	char *safe_pathname;
+	int len, ret;
+	
+	safe_pathname = g_strdup (pathname);
+	len = strlen (safe_pathname);
+	
+	if (len > 1 && G_IS_DIR_SEPARATOR (safe_pathname[len - 1]))
+		safe_pathname[len - 1] = '\0';
+
+	ret = g_mkdir (safe_pathname, mode);
+
+	g_free (safe_pathname);
+
+	return ret;
+}
+
 static void
 libgnome_userdir_setup (gboolean create_dirs)
 {
@@ -280,7 +299,11 @@ libgnome_userdir_setup (gboolean create_dirs)
 
                         /* chop trailing slash */
                         len = strlen (gnome_user_dir);
-                        if (len > 1 && gnome_user_dir[len-1] == '/')
+                        if (len > 1 && (gnome_user_dir[len-1] == G_DIR_SEPARATOR
+#ifdef G_OS_WIN32
+					|| gnome_user_dir[len-1] == '/'
+#endif
+					))
                                 gnome_user_dir[len - 1] = '\0';
                         
                         gnome_user_private_dir = g_strconcat (gnome_user_dir,
@@ -299,24 +322,24 @@ libgnome_userdir_setup (gboolean create_dirs)
 	if (!create_dirs)
 		return;
 	
-	if (mkdir (gnome_user_dir, 0700) < 0) { /* private permissions, but we
-						   don't check that we got them */
+	if (safe_mkdir (gnome_user_dir, 0700) < 0) { /* private permissions, but we
+							don't check that we got them */
 		if (errno != EEXIST) {
-			fprintf(stderr, _("Could not create per-user gnome configuration directory `%s': %s\n"),
+			g_printerr (_("Could not create per-user gnome configuration directory `%s': %s\n"),
 				gnome_user_dir, strerror(errno));
 			exit(1);
 		}
 	}
     
-	if (mkdir (gnome_user_private_dir, 0700) < 0) { /* This is private
-							   per-user info mode
-							   700 will be
-							   enforced!  maybe
-							   even other security
-							   meassures will be
-							   taken */
+	if (safe_mkdir (gnome_user_private_dir, 0700) < 0) { /* This is private
+								per-user info mode
+								700 will be
+								enforced!  maybe
+								even other security
+								meassures will be
+								taken */
 		if (errno != EEXIST) {
-			fprintf (stderr, _("Could not create per-user gnome configuration directory `%s': %s\n"),
+			g_printerr (_("Could not create per-user gnome configuration directory `%s': %s\n"),
 				 gnome_user_private_dir, strerror(errno));
 			exit(1);
 		}
@@ -325,14 +348,14 @@ libgnome_userdir_setup (gboolean create_dirs)
 
 	/* change mode to 0700 on the private directory */
 	if (chmod (gnome_user_private_dir, 0700) < 0) {
-		fprintf(stderr, _("Could not set mode 0700 on private per-user gnome configuration directory `%s': %s\n"),
+		g_printerr (_("Could not set mode 0700 on private per-user gnome configuration directory `%s': %s\n"),
 			gnome_user_private_dir, strerror(errno));
 		exit(1);
 	}
   
-	if (mkdir (gnome_user_accels_dir, 0700) < 0) {
+	if (safe_mkdir (gnome_user_accels_dir, 0700) < 0) {
 		if (errno != EEXIST) {
-			fprintf(stderr, _("Could not create gnome accelerators directory `%s': %s\n"),
+			g_printerr (_("Could not create gnome accelerators directory `%s': %s\n"),
 				gnome_user_accels_dir, strerror(errno));
 			exit(1);
 		}
@@ -418,6 +441,7 @@ gnome_vfs_module_info_get (void)
 	};
 	return &module_info;
 }
+
 /**
 * libgnome_module_info_get:
 * 
@@ -436,27 +460,30 @@ libgnome_module_info_get (void)
 		gnomelib_options,
 		NULL, NULL, NULL, NULL
 	};
+	int i = 0;
 
 	if (module_info.requirements == NULL) {
 		static GnomeModuleRequirement req[4];
 
 		bindtextdomain (GETTEXT_PACKAGE, LIBGNOME_LOCALEDIR);
-#if 0
 #ifdef HAVE_BIND_TEXTDOMAIN_CODESET
 		bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 #endif
-#endif
-		req[0].required_version = "0.9.1";
-		req[0].module_info = gnome_bonobo_activation_module_info_get ();
+		req[i].required_version = "0.9.1";
+		req[i].module_info = gnome_bonobo_activation_module_info_get ();
+		i++;
 
-		req[1].required_version = "0.3.0";
-		req[1].module_info = gnome_vfs_module_info_get ();
+		req[i].required_version = "0.3.0";
+		req[i].module_info = gnome_vfs_module_info_get ();
+		i++;
 
-		req[2].required_version = "1.1.1";
-		req[2].module_info = gnome_gconf_module_info_get ();
+		req[i].required_version = "1.1.1";
+		req[i].module_info = gnome_gconf_module_info_get ();
+		i++;
 
-		req[3].required_version = NULL;
-		req[3].module_info = NULL;
+		req[i].required_version = NULL;
+		req[i].module_info = NULL;
+		i++;
 
 		module_info.requirements = req;
 	}
